from h5py import Dataset, File, Group
from gwpy.frequencyseries import FrequencySeries
from .section import Section
from .metadata.root import Root as Metadata
from .utils import (
    frequencyseries_to_dataset,
    dataset_to_frequencyseries,
)
from .scatterer import Scatterer


class Root(Section):
    """
    Simulation data identified with an unique name, with information
    about the code that generated it
    """

    def __init__(
        self,
        metadata: Metadata,
        scatterers: list[Scatterer],
        darm: FrequencySeries,
    ):
        """
        Parameters
        ----------
        metadata : Metadata
            Metadata associated to this data package
        scatterers: list[Scatterer]
            Scatterers data list
        darm : FrequencySeries
            common DARM transfer function for all this simulation data
        """
        self.metadata = metadata
        self.scatterers = scatterers
        self.darm = darm

    @staticmethod
    def from_hdf5(store: File | Group) -> "Root":
        scatterers_group = store.get("scatterers")
        darm_dataset = store.get("DARM")
        if not isinstance(scatterers_group, Group):
            raise ValueError(
                "scatterers group should exists, and not be {}".format(
                    type(scatterers_group)
                )
            )
        if not isinstance(darm_dataset, Dataset):
            raise ValueError(
                "DARM dataset should exists, and not be {}".format(
                    type(darm_dataset)
                )
            )
        scatterers = scatterers_group.values()
        return Root(
            metadata=Metadata.from_hdf5(store),
            scatterers=[
                Scatterer.from_hdf5(scatterer)
                for scatterer in scatterers
            ],
            darm=dataset_to_frequencyseries(darm_dataset),
        )

    def to_hdf5(self, store: File | Group) -> File | Group:
        _ = self.metadata.to_hdf5(store)
        for scatterer in self.scatterers:
            _ = scatterer.to_hdf5(
                store.require_group("scatterers").create_group(
                    scatterer.metadata.name,
                )
            )
        _ = frequencyseries_to_dataset(store, "DARM", self.darm)
        return store
