from astropy.units import Unit, Quantity
from h5py import Dataset, File, Group
from gwpy.frequencyseries import FrequencySeries


def dataset_to_frequencyseries(dataset: Dataset) -> FrequencySeries:
    """
    Convert a h5df dataset to a FrequencySeries keeping the units right
    Parameters
    ----------
    dataset : Dataset
        Dataset to convert
    Returns
    -------
    TimeSeries : TimeSeries with a good unit
    """
    unit = None
    if "units" in dataset.attrs.keys():
        units = dataset.attrs["units"]
        if isinstance(units, str):
            unit = Unit(units)
        else:
            raise ValueError(
                "cannot convert {} to a unit".format(units)
            )
    if "f0" not in dataset.attrs.keys():
        raise ValueError(
            "no starting time defined for this frequencyseries"
        )
    if "df" not in dataset.attrs.keys():
        raise ValueError(
            "no time sample separation defined for this frequencyseries"
        )
    f0 = (dataset.attrs["f0"],)
    df = dataset.attrs["df"]
    if not isinstance(f0, str):
        raise ValueError(
            "f0 must be stored as string, not {}".format(type(f0))
        )
    if not isinstance(df, str):
        raise ValueError(
            "df must be stored as string, not {}".format(type(df))
        )
    f0 = Quantity(f0)
    df = Quantity(df)
    return FrequencySeries(
        data=dataset[1],
        unit=unit,
        f0=f0,
        df=df,
    )


def frequencyseries_to_dataset(
    store: File | Group, name: str, frequencyseries: FrequencySeries
) -> Dataset:
    """
    Convert a gwpy FrequencySeries to a hdf5 dataset keeping the units right
    Parameters
    ----------
    store : File | Group
        hdf5 element which will store the Dataset
    name : str
        Name of the dataset in the store
    timeseries : TimeSeries
        Data to store in the Dataset
    Returns
    -------
    Dataset : Created dataset
    """
    dataset = store.create_dataset(name, data=frequencyseries.data)
    if frequencyseries.unit is not None:
        dataset.attrs["units"] = frequencyseries.unit.to_string()
    if not isinstance(frequencyseries.t0, Quantity):
        raise ValueError(
            "t0 must be a quantity, not {}".format(
                type(frequencyseries.f0)
            )
        )
    if not isinstance(frequencyseries.dt, Quantity):
        raise ValueError(
            "t0 must be a quantity, not {}".format(
                type(frequencyseries.df)
            )
        )
    dataset.attrs["f0"] = frequencyseries.t0.to_string()
    dataset.attrs["df"] = frequencyseries.dt.to_string()
    return dataset
